// Copyright 2017 The Johns Hopkins University Applied Physics Laboratory.
// Licensed under the MIT License. See LICENSE.txt in the project root for full license information.

#include <tclap/CmdLine.h>
#include <eopcc/cloud_io.hpp>
#include <eopcc/utm_zone.hpp>
#include <gdal_priv.h>



int main(int argc, char* argv[]) {
  try {
    TCLAP::CmdLine cmd("read-kml command line", ' ', "1.0");

    TCLAP::ValueArg<std::string> input_filename("i","input", "input kml", true,  "", "kml file");
    TCLAP::ValueArg<std::string> laz_filename("l", "laz", "input laz", false, "", "laz file");
    cmd.add(input_filename);
    cmd.add(laz_filename);
    cmd.parse(argc, argv);

    std::string input_file = input_filename.getValue();
    std::string laz_file = laz_filename.getValue();
    eopcc::PolygonExtent extent =  eopcc::ExtentFromKML(input_file);
    if (!laz_file.empty()) {
      eopcc::PointCloud pc;
      eopcc::Load(laz_file, pc);

      std::cout << pc.extent << std::endl;
      std::cout << pc.calculate_extent() << std::endl;

      if (pc.wkt_spatial_ref == "") {
	      std::cerr << "No spatial reference included in the point cloud (maybe PLY?)" << std::endl;
	      std::cerr << "Assuing point cloud is the same UTM zone as the KML file (and that the points are in  easting,northing,alt)" << std::endl;
      } else {
	      double lon = extent.sides[0].start.x;
	      double lat = extent.sides[0].start.y;
	      extent.MatchCoordinatesUTM(GetUTMZone(lat, lon),
	                                 IsNorthernHemisphere(lat));
      }



      //eopcc::Extent box_extent = extent.to_extent();
      std::cout << extent.to_extent() << std::endl;

      int count = pc.inside_extent(extent);

      std::cout << "Contains " << count << "/" << pc.size() << std::endl;

    } else {
      for (auto line : extent.sides) {
        std::cout << line.start << " -> " << line.end << std::endl;
      } 
    }
  } catch (std::exception& e) {
    std::cerr << "error: " << e.what() << std::endl;
    return EXIT_FAILURE;
  }

  return EXIT_SUCCESS;
}
